/************************************************************************/
/*                                                                      */
/*                      Debouncing 8 Keys                               */
/*                      Sampling 4 Times                                */
/*                      With Repeat Function                            */
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/
 
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "cpu.h"
#include "i2cmaster.h"
 
#ifndef F_CPU
	#warning F_CPU not defined
#endif

#include "i2c_key.h"
 
#ifndef FALSE
	#define FALSE (char)0
	#define TRUE (char)!FALSE
#endif

volatile uint8_t key_state=0;                                // debounced and inverted key state:
                                                  // bit = 1: key pressed
volatile uint8_t key_press=0;                                // key press detect

volatile uint8_t key_rpt=0;                                  // key long press and repeat
 
volatile uint8_t _i2c_key=0;

void timer2_init(void)
{	
#ifdef F_CPU8000000
	//OCR2 = 125;	// Reloadwert Timer 2
	//OCR2 = 128;	// fast erreicht Reloadwert Timer 2 rechnerisch richtig wr 125, aber weil zu schnell s.o. 
	//OCR2A = 76;	// Reloadwert Timer 2 rechnerisch richtig wr 78,125, aber weil zu schnell s.o. man knnte alle 8 auf 79 reload
	OCR2A = 76;	// Reloadwert Timer 2 rechnerisch richtig wr 78,125, aber weil zu schnell s.o. man knnte alle 8 auf 79 reload
	//#warning hier richtig2
#endif
#ifdef F_CPU4000000	
	OCR2A = 63;	// Reloadwert Timer 2 rechnerisch richtig wr 78,125, aber weil zu schnell s.o. man knnte alle 8 auf 79 reload
	#warning hier falsch
#endif

	TCCR2A = (1<<WGM21) | (1<<COM2B1);	// xxxx2 meens timer2 !
	TCCR2B = (1<<CS22) | (1<<CS21) | (1<<CS20);	// xxxx2 meens timer2 !
	// WGM21 	-> 	Clear Timer (on) Compare CTC mode / COM21 ->Clear OC2 on compare
	// CS22 	->	clkT2S/64 (From prescaler) Takt= 8MHz/64
	/*
	CS22 	CS21 	CS20 	Description
	0 		0 		0 		No clock source (Timer/Counter stopped).
	0 		0 		1 		clkT2S/(No prescaling)
	0 		1 		0 		clkT2S/8 (From prescaler)
	0 		1 		1 		clkT2S/32 (From prescaler)
	1 		0 		0 		clkT2S/64 (From prescaler)
	1 		0 		1 		clkT2S/128 (From prescaler)
	1 		1 		0 		clkT2S/256 (From prescaler)
	1 		1 		1 		clkT2S/1024 (From prescaler)
	*/
	TIMSK2 = (1 << OCIE2A);
}


uint8_t test_i2c_key(void)
{	uint8_t dummy;    
	_i2c_key=0;
	
	if(!i2c_start(PCF8574A_0+I2C_READ))	//;	// set device address and write mode
	{	dummy=i2c_readNak();
		_i2c_key='A';
	}
	else 
	{	if(!i2c_start(PCF8574_0+I2C_READ))	//;	// set device address and write mode
		{	dummy=i2c_readNak();
			_i2c_key=' ';
		}
	}
	i2c_stop();
	return _i2c_key;
}

///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed. Each pressed key is reported
// only once
//
uint8_t get_key_press( uint8_t key_mask )
{	cli();                                          // read and clear atomic !
  	key_mask &= key_press;                          // read key(s)
  	key_press ^= key_mask;                          // clear key(s)
	sei();
  	return (key_mask);
}
 
///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported beeing pressed in subsequent calls
// to this function. This simulates the user repeadiately
// pressing and releasing the key.
//
uint8_t get_key_rpt( uint8_t key_mask )
{ 	cli();                                          // read and clear atomic !
  	key_mask &= key_rpt;                            // read key(s)
  	key_rpt ^= key_mask;                            // clear key(s)
  	sei();
  	return (key_mask);
}
 
///////////////////////////////////////////////////////////////////
//
uint8_t get_key_short( uint8_t key_mask )
{
  	cli();                                          // read key state and key press atomic !
  	return get_key_press( ~key_state & key_mask );
}
 
///////////////////////////////////////////////////////////////////
//
uint8_t get_key_long( uint8_t key_mask )
{
  	return get_key_press( get_key_rpt( key_mask ));
}

