#ifndef RTC_H
#define RTC_H	1

#ifndef MY_TYPES_H_
	#include "my_types.h"
#endif
#ifndef I2CMASTER_H
#include "i2cmaster.h"
#endif

#define CENTURY 	7

#include "clock.h"

extern BOOL	_test_i2c_rtc;
extern BOOL	_time_i2c_rtc;
extern char _uhr_string[];
extern BOOL	_uhr_go;

extern volatile UBYTE _ms10;

extern UBYTE _rtc_t[3];

extern UBYTE _rtc_d[5];

#define DS3231  			0xD0	// deviceaddress RTC DS3231, see datasheet
// ADDRESS
#define SECONDS				0	// 00H Seconds 00-59
#define MINUTES				1	// 01H Minutes 00-59
#define HOURS					2	// 02H Hours 00-23 od. 1-12 + !AM/PM
#define DAY						3  // 03H Day 1-7
#define DATE					4 	// 04H Date 00-31
#define C_MONTH				5	// 05H Century Month 1-12 + Century
#define CENTURY				7	// Century Bit in Month
#define YEAR					6	// 06H Year 00-99
#define A1_SECONDS			7	// 07H A1M1 Alarm 1 Seconds 00-59
#define A1_MINUTES			8	//	08H A1M2 Alarm 1 Minutes 00-59
#define A1_HOURS 				9	// 09H A1M3 12/!24 Alarm 1 Hours 00-23 od. 1-12 + !AM/PM
#define A1M4					10	// 0AH A1M4 DY/!DT Alarm 1 Day 1-7 od. Alarm 1 Date 1-31
#define A2_MINUTES			11	// 0BH A2M2 Alarm 2 Minutes 00-59
#define A2_HOURS				12	//	0CH A2M3 12/!24 Alarm 2 Hours 00-23 od. 1-12 + !AM/PM
#define A2M4					13	// 0DH A2M4 DY/!DT Alarm 2 Day 1-7 od. Alarm 2 Date 1-31
#define DS3231_ControlReg	14 // RTC DS3231_ControlRegister (0Eh)
// BITS
#define EOSC 					7
#define BBSQW 					6
#define CONV 					5
#define RS2 					4
#define RS1						3
#define INTCN					2
#define A2IE					1
#define A1IE					0

#define DS3231_StatusReg	15 // RTC DS3231_StatusRegister (0Fh)
// BITS
#define OSF						7 
#define EN32kHz				3 
#define BSY						2 
#define A2F						1 
#define A1F						0

#define AGING_OFFSET			16 // 10H SIGN DATA DATA DATA DATA DATA DATA DATA Aging Offset
#define MSB_TEMP				17 // 11H SIGN DATA DATA DATA DATA DATA DATA DATA MSB of Temp
#define LSB_TEMP				18 // 12H DATA DATA 0 0 0 0 0 0 LSB of Temp

extern PROGMEM const char *mo_table[];

struct _TIME_FORMAT {
	UBYTE		seconds;		//!< Sekunden
	UBYTE		minutes;		//!< Minuten
	UBYTE		hours;		//!< Stunden
  	UBYTE 		wt;
	char		_time_[9];
	char		_build_[9];
	UBYTE		flags;		// 7=i2c_rtc, 6=i2c_rtc_true, 5=dcf77, 4=dcf77_true
};

struct _DATE_FORMAT {
	UBYTE		tt;
	UBYTE		mo;
	uint16_t		jjjj;
	UBYTE		wt;
	char		_date_[11];
	char		_build_[12];
	char		_wochentag_[11];
	UBYTE		flags;		// 7=i2c_rtc, 6=i2c_rtc_true, 5=dcf77, 4=dcf77_true
};

extern struct _TIME_FORMAT softclock, rtctime, dcf77time;
extern struct _DATE_FORMAT	softdate, rtcdate, dcf77date;

BOOL test_i2c_rtc(void);
char *time_str_rtc(void);
char *date_str_rtc(BOOL); // TRUE jahr 4-stellig -> FALSE jahr 2-stellig
char *sh_dt_str_rtc(void);
char *sh_dt_str_build(void);

extern char t_bld[], t_rtc[];

void write_rtc( UBYTE, UBYTE );
char *temperatur_str_rtc(BYTE);
char *wt_str_rtc(void);
void _clock(void);
void clr_i2c_rtc(void);
void read_rtcTD(void);

void alarm1_set_rel_sec_max65536(UWORD /*plus_sekunden*/);
void alarm2_set_rel_min_max65536(UWORD /*plus_minuten*/);
void alarm1_set_abs_rtc(UBYTE /*sekunde*/, UBYTE /*minute*/, UBYTE /*stunde*/, UBYTE /*tag*/);
void alarm2_set_abs_rtc(UBYTE /*minute*/, UBYTE /*stunde*/, UBYTE /*tag*/);

void alarm1_clr_rtc(void);
void alarm2_clr_rtc(void);
UBYTE test_status_rtc(void);
UBYTE shutdown(UBYTE);
UBYTE sekunde_rtc(void);
UBYTE minute_rtc(void);
void init_softclock(void);

#endif // #ifndef RTC_H
