#ifndef MY_TYPES_H
	#include "my_types.h"
#endif

#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include "bits.h"
#include "calen.h"
#include "cpu.h"
#include "ctype.h"
#include "i2c_key.h"
#include "i2c_rtc.h"
#include "i2cmaster.h"
#include "lcd6.h"
#include "s_tools.h"

extern void lcd_set(void);
extern volatile unsigned char _break_write;
extern volatile UBYTE save_rtc_status;
extern volatile UBYTE __hour;
extern volatile UBYTE __minute;
extern volatile UBYTE __day;
extern volatile UBYTE __month;
extern volatile UWORD __year;

extern char __buildtime__[];
extern char __builddate__[];

struct _TIME_FORMAT softclock, rtctime, dcf77time;
struct _DATE_FORMAT	softdate, rtcdate, dcf77date;

BOOL 	_test_i2c_rtc	=	FALSE;
BOOL 	_time_i2c_rtc	=	FALSE;
BOOL	_uhr_go			=	FALSE;
char t_bld[15]={0}, t_rtc[15]={0};

#define ALARM_int1_DDR		DDRD
#define ALARM_int1_PORT 	PORTD		// port for DCF_LED
#define _ALARM_int1_pin		3       	// pin  for DCF_LED

extern volatile UBYTE __p;
extern UBYTE kon, hell;

char _uhr_string[11]="leer";
ULONG time1_;
ULONG time2_;

#define DS3231  	0xD0		// device address of RTC DS3231, see datasheet
//#define PCF8574_0  	0x40  // device address of PCF8574, see datasheet
//#define PCF8574A_0  	0x70	// device address of PCF8574A, see datasheet

UBYTE _rtc_t[3]={0};
UBYTE _rtc_d[5]={0};


double _rtc_temp=0.0;

extern BOOL soft_update;
extern volatile BOOL war_alarm;
extern char rtc_string[41];

void all_led(UBYTE);

char *time_str_rtc(void);
char *date_str_rtc(BOOL);
char *temperatur_str_rtc(BYTE);

void clr_i2c_rtc(void)
{	UBYTE tmp=7;
	if(!_i2c_init)
		i2c_init();
	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);     // set device address and write mode
  	i2c_write(SECONDS);                   // write address = SECONDS
	while(tmp--)
		i2c_write(0);
	i2c_stop();
}

void read_rtcTD(void)
{	if(!_i2c_init)
		i2c_init();
	while(_i2c_busy);
	if(!i2c_start(DS3231+I2C_WRITE))				// set device address and write mode
	{	do
		{	i2c_write(SECONDS);						// write address = SECONDS
			i2c_rep_start(DS3231+I2C_READ);     	// set device address and read mode
			_rtc_t[2]=bcdtoint(i2c_readAck()&0x7f);	// sekunden
			_rtc_t[1]=bcdtoint(i2c_readAck()&0x7f); // minuten
			_rtc_t[0]=bcdtoint(i2c_readAck()&0x3f); // stunden
			_rtc_d[4]=i2c_readAck()&7;				// wt
			_rtc_d[0]=bcdtoint(i2c_readAck()&0x3f);	// tag
			_rtc_d[1]=i2c_readAck();				// monat mit CENTURY
			_rtc_d[2]=i2c_readNak();				// jahr
   			i2c_stop();
			if(_rtc_d[1] & (1<<CENTURY))
				_rtc_d[3]=1;
			else
				_rtc_d[3]=0;
			_rtc_d[2] &= ~(1<<CENTURY); 
			_rtc_d[2] = bcdtoint(_rtc_d[2]);
			_rtc_d[1]=bcdtoint(_rtc_d[1]&0x1f);		// monat nach CENTURY
		}while(_rtc_t[2]==59);						// sicherheitshalber nicht in Sekunde 59 !
		__hour=_rtc_t[0];
		__minute=_rtc_t[1];
		__day=_rtc_d[0];
		__month=_rtc_d[1];
		__year=1900+_rtc_d[3]*100+_rtc_d[2];

	} // if(!i2c_start(DS3231+I2C_WRITE))
} // void read_rtcTD(void)


char *sh_dt_str_build(void)
{	static char _sh_build_dt_str[15];
	char s[4]={0},	*sh_p_str;
	UBYTE i=12;	
	strcpy(_sh_build_dt_str,rights(__builddate__,4));
	while(i>0)
	{  if((strcmp_P(lefts(__builddate__, 3),(char*)pgm_read_word(&(mo_table[i]))))) // Nein, dann weitersuchen
	  		i--;
		else
		break;
	}
	strcat(_sh_build_dt_str,trimm_string('0',itoa(i,s,10),2));
	sh_p_str=&__builddate__[0];
	strcpy(s,"0");
	while(!isdigit(*sh_p_str))
		sh_p_str++;
	while(isdigit(*sh_p_str))
		strncat(s,sh_p_str++,1);
	strcat(_sh_build_dt_str,rights(s,2));
	strcat(_sh_build_dt_str,trimm_string('0',lefts(__buildtime__,2),2));
	strcat(_sh_build_dt_str,trimm_string('0',mids(__buildtime__,4,2),2));
	strcat(_sh_build_dt_str,trimm_string('0',rights(__buildtime__,2),2));
	return _sh_build_dt_str;
}


char *sh_dt_str_rtc(void)
{	static char _rtc_dt_str[15];

	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);			// set device address and write mode
	do
	{	i2c_write(SECONDS);						// write address = SECONDS
		i2c_rep_start(DS3231+I2C_READ);     	// set device address and read mode
		_rtc_t[2]=i2c_readAck()&0x7f;			// Seconds
		_rtc_t[1]=i2c_readAck()&0x7f;			// Minutes
		_rtc_t[0]=i2c_readNak()&0x3f;			// Hours
		i2c_stop();
	}while(bcdtoint(_rtc_t[2])==59);						// sicherheitshalber nicht in Sekunde 59 !

	_rtc_dt_str[8]=(_rtc_t[0]>>4)+'0';
	_rtc_dt_str[9]=(_rtc_t[0]&0xf)+'0';
	_rtc_dt_str[10]=(_rtc_t[1]>>4)+'0';
	_rtc_dt_str[11]=(_rtc_t[1]&0xf)+'0';
	_rtc_dt_str[12]=(_rtc_t[2]>>4)+'0';
	_rtc_dt_str[13]=(_rtc_t[2]&0xf)+'0';
	_rtc_dt_str[14]=0;

	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(DATE);

	i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
	_rtc_d[0]=i2c_readAck()&0x3f;
	_rtc_d[1]=i2c_readAck()&0x1f;
	_rtc_d[2]=i2c_readNak();
	i2c_stop();

	_rtc_dt_str[0]='2';
	_rtc_dt_str[1]='0';
	_rtc_dt_str[2]=(_rtc_d[2]>>4)+'0';
	_rtc_dt_str[3]=(_rtc_d[2]&0xf)+'0';
	_rtc_dt_str[4]=(_rtc_d[1]>>4)+'0';
	_rtc_dt_str[5]=(_rtc_d[1]&0xf)+'0';
	_rtc_dt_str[6]=(_rtc_d[0]>>4)+'0';
	_rtc_dt_str[7]=(_rtc_d[0]&0xf)+'0';

	return _rtc_dt_str;
}

BOOL test_i2c_rtc(void)
{	
	//char t_bld[15]={0}, t_rtc[15]={0};
	char *p_rtc;

	UBYTE v1=0, v2=0, control;
	if(!_i2c_init)
		i2c_init();

	_test_i2c_rtc=FALSE;
	strcpy((char *)_uhr_string, "ungestellt");
	_uhr_go=FALSE;

	read_rtcTD();

	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(MINUTES);
   i2c_write(0x55);                    // write value 0x55 to i2c Minutes
   i2c_write(0x06);                    // write value 0x6 to i2c Hours
   i2c_stop();     
	
	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(MINUTES);

	i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
	v1=i2c_readAck();
	v2=i2c_readNak();
   i2c_stop();     

	if(v1==0x55)
	{	if(v2==0x06)
		{	_test_i2c_rtc=TRUE;
			while(_i2c_busy);
			i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   		i2c_write(DS3231_ControlReg);
			i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
			control = (i2c_readNak() & ( 1<<A1IE | 1<<A2IE )) | 1<<INTCN; // nur INT setzen AIE lassen
    		i2c_stop();
				
			while(_i2c_busy);
			i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   		i2c_write(DS3231_ControlReg);
			i2c_write( control );
			i2c_stop();

			while(_i2c_busy);
			i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
		   i2c_write(MINUTES);                 // write address = 1 (Minutenreg)
		   i2c_write(int2bcd(_rtc_t[1]));      // write restore
		   i2c_write(int2bcd(_rtc_t[0]));      // write restore
		   i2c_stop();     

			_time_i2c_rtc = 0;				
			strcpy(t_rtc, sh_dt_str_rtc());
			strcpy(t_bld, sh_dt_str_build());

			if(strcmp(t_rtc, t_bld)>0)
				_time_i2c_rtc = 1;

			if(_time_i2c_rtc)
			{	strcpy(t_rtc, sh_dt_str_rtc());
				p_rtc=t_rtc;
				while(*p_rtc)
					_time_i2c_rtc = isdigit(*p_rtc++);				
				if(_time_i2c_rtc)
					if(!strcmp(t_bld,t_rtc)<0)
						_time_i2c_rtc=FALSE;
					
			}

			if(!_time_i2c_rtc)
			{	while(_i2c_busy);
				i2c_start_wait(DS3231+I2C_WRITE);     // set device address and write mode
		   	i2c_write(SECONDS);                   // write address = SECONDS
				i2c_write(int2bcd(atoi(rights(t_bld, 2))));
				i2c_write(int2bcd(atoi(mids(t_bld, 11, 2))));
				i2c_write(int2bcd(atoi(mids(t_bld, 9, 2))));
		
				//UWORD tagesnummer( UWORD jahr, UBYTE monat, UBYTE tag )
				//UBYTE wochentag_im_jahr(UWORD jahr, UWORD n) // Eingabe:  Jahr, Tagesnummer // Ausgabe:  WT (1=Mo,..., 6=Sa, 7=So)

				i2c_write(wochentag_im_jahr(atoi(lefts(t_bld,4)),tagesnummer(atoi(lefts(t_bld,4)),atoi(mids(t_bld, 5, 2)),atoi(mids(t_bld, 7, 2)))));
				i2c_write(int2bcd(atoi(mids(t_bld, 7, 2)))); // tag
				i2c_write(int2bcd(atoi(mids(t_bld, 5, 2)))|(1<<CENTURY));
				i2c_write(int2bcd(atoi(mids(t_bld, 3, 2))));
	
				i2c_stop();
			} // if(!_time_i2c_rtc)

		} // if(v2==0x06)
	} // if(v1==0x55)
	return _test_i2c_rtc;
}

/*
SIGNAL (SIG_PIN_CHANGE3)
{	UBYTE nix;
	nix=1;
	PCMSK3&=~(1<<PCINT27);
	PCICR&=~(1<<PCIE3);
	PCIFR=(1<<PCIF3);
	//war_alarm=TRUE;
}
*/

UBYTE test_status_rtc(void)
{	static UBYTE status;
	status=0;
	if(_test_i2c_rtc)
	{	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
	  	i2c_write(DS3231_StatusReg);       // write address = DS3231_ControlReg
		i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
		status = i2c_readNak() & ( (1<<A1F)|(1<<A2F) );			// 0x03
		status<<=6;
		i2c_stop();
	}
	return status;
}


void alarm1_clr_rtc(void)
{	UBYTE control, status;
	if(_test_i2c_rtc)
	{	i2c_start_wait(DS3231+I2C_WRITE);							// set device address and write mode
		i2c_write(DS3231_ControlReg);       						// write address = DS3231_ControlReg
		i2c_rep_start(DS3231+I2C_READ);     						// set device address and read mode
		control = (i2c_readAck()&( 1<<A2IE )) | ( 1<<INTCN );	// nur A1IE lschen
		status = i2c_readNak() & ~(1<<A1F);							// nur A1F lschen
		i2c_stop();

		i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
		i2c_write(DS3231_ControlReg);       // write address = DS3231_ControlReg
		i2c_write( control );
		i2c_write( status );
		i2c_stop();
	}
}


void alarm2_clr_rtc(void)
{	UBYTE control, status;
	if(_test_i2c_rtc)
	{	i2c_start_wait(DS3231+I2C_WRITE);							// set device address and write mode
		i2c_write(DS3231_ControlReg);       						// write address = DS3231_ControlReg
		i2c_rep_start(DS3231+I2C_READ);     						// set device address and read mode
		control = (i2c_readAck()&( 1<<A1IE )) | ( 1<<INTCN );	// nur A2IE lschen
		status = i2c_readNak() & ~(1<<A2F);							// nur A2F lschen
		i2c_stop();

		i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
		i2c_write(DS3231_ControlReg);       // write address = DS3231_ControlReg
		i2c_write( control );
		i2c_write( status );
		i2c_stop();
	}
}


UBYTE shutdown(UBYTE _sleep_mode)
{	static UBYTE wakeup_from, hell;
	wakeup_from=0;

	cli();
	while(get_key_press(1<<KEY_ENTER));
	_delay_up25(4);

	set_sleep_mode(_sleep_mode);
	PCMSK3|=(1<<PCINT27);
	PCICR|=(1<<PCIE3);
				   
	EIMSK=0;
	EIFR=255;

  	CLS();
	
	if(_sleep_mode==SLEEP_MODE_PWR_DOWN)
	{
		if(!(ALARM_int1_DDR & ~(1<<_ALARM_int1_pin)))
			ALARM_int1_DDR &= ~(1<<_ALARM_int1_pin);		// ALARM_int1_DDR auf in
		ALARM_int1_PORT |= (1<<_ALARM_int1_pin);			// ALARM_int1_PORT pullup enable

		all_led(_OFF);

	#ifdef HELL_DDR	
		hg_led_set(_OFF);
	#endif
		lcd_init(LCD_DISP_OFF); // LCD_DISP_OFF
		//VCC_LCD_PORT &= ~(1<<_vccLCD);    // XOR, Kurzschreibweise, PORTB = PORTB ^ (1<<PB0) 
	}

/*
	if(_sleep_mode==SLEEP_MODE_IDLE)
	{	gruenLED(1);
	}
*/
	sleep_enable();

  	sei();
  	sleep_cpu();
	_delay_up25(4);
	
	sleep_disable();
	
	cli();
	gruenLED(1);   	// _POWER_ON LED an	

	//alarm1_clr_rtc();
	PCIFR=(1<<PCIF3);
	PCMSK3&=~(1<<PCINT27);
	PCICR&=~(1<<PCIE3);
	TIMSK0 &=~(1<<TOIE0);		//disable timer interrupt
	TIMSK2 = 0;
	EIMSK&=~((1<<INT1)|(1<<INT0));
	EIFR&=~((1<<INTF1)|(1<<INTF0));
  	
	//sei();

	if((wakeup_from=test_status_rtc())>0)
	{	save_rtc_status=wakeup_from;
		(wakeup_from&0x40) ? alarm1_clr_rtc() : alarm2_clr_rtc();
	}
#ifdef KEY_ESC
	else
		wakeup_from=(1<<KEY_ESC);
#endif

   if( !(HELL_DDR	& (1 << _hellLED_PIN)) )
	{
#ifdef HELL_DDR	
		HELL_DDR	|= (1 << _hellLED_PIN);		// OC1A DDR out
		hell=old_hell;
		hg_led_set(hell);
#endif
#ifdef KON_DDR
		OCR1BL = kon; // OCR1BL
else
		kon=(((_KON_MIN-_KON_MAX)/2)+_KON_MAX); //_KON_PEEK * kon / 100
#endif
#ifdef VCC_LCD_PORT
		VCC_LCD_PORT |= (1<<_vccLCD);    // XOR, Kurzschreibweise, PORTB = PORTB ^ (1<<PB0) 
#endif
	all_led(RESET);
	all_led(25);
_delay_ms(250);
	all_led(25);
_delay_ms(250);
	all_led(25);
_delay_ms(250);
	}

	EIMSK|=((1<<INT0));

	//POWER_LED_PORT 	|= (1 << _powerLED);
	return(wakeup_from);
}


UBYTE Monatstage( UWORD, UBYTE );

char *time_str_rtc(void)
{	static char _rtc_time_str[9];

	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(SECONDS);                 // write address = Seconds

	i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
	_rtc_t[2]=i2c_readAck()&0x7f;			// Seconds
	_rtc_t[1]=i2c_readAck()&0x7f;			// Minutes
	_rtc_t[0]=i2c_readNak()&0x3f;			// Hours
	i2c_stop();

	_rtc_time_str[0]=(_rtc_t[0]>>4)+'0';
	_rtc_time_str[1]=(_rtc_t[0]&0xf)+'0';
	_rtc_time_str[2]=':';
	_rtc_time_str[3]=(_rtc_t[1]>>4)+'0';
	_rtc_time_str[4]=(_rtc_t[1]&0xf)+'0';
	_rtc_time_str[5]=':';
	_rtc_time_str[6]=(_rtc_t[2]>>4)+'0';
	_rtc_time_str[7]=(_rtc_t[2]&0xf)+'0';
	_rtc_time_str[8]=0;

	return _rtc_time_str;
}


char *date_str_rtc(BOOL wie)
{	static char _rtc_date_str[11];

	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(DATE);

	i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
	_rtc_d[0]=i2c_readAck()&0x3f;
	_rtc_d[1]=i2c_readAck();
	_rtc_d[2]=i2c_readNak();
	i2c_stop();

	_rtc_date_str[0]=(_rtc_d[0]>>4)+'0';
	_rtc_date_str[1]=(_rtc_d[0]&0xf)+'0';
	_rtc_date_str[2]='.';
	_rtc_date_str[3]=((_rtc_d[1]&(0x3f))>>4)+'0';
	_rtc_date_str[4]=(_rtc_d[1]&0xf)+'0';
	_rtc_date_str[5]='.';
	if(wie)
	{	if(_rtc_d[1]&(1<<CENTURY))
		{	_rtc_date_str[6]='2';
			_rtc_date_str[7]='0';
		}
		else	
		{	_rtc_date_str[6]='1';
			_rtc_date_str[7]='9';
		}
		_rtc_date_str[8]=(_rtc_d[2]>>4)+'0';
		_rtc_date_str[9]=(_rtc_d[2]&0xf)+'0';
		_rtc_date_str[10]=0;
	}
	else
	{	_rtc_date_str[6]=(_rtc_d[2]>>4)+'0';
		_rtc_date_str[7]=(_rtc_d[2]&0xf)+'0';
		_rtc_date_str[8]=0;
	}
	return _rtc_date_str;
}


void write_rtc( UBYTE _adress, UBYTE _byte )
{	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);		// set device address and write mode
   i2c_write(_adress);
   i2c_write(_byte);
   i2c_stop();                             
}


char *temperatur_str_rtc(BYTE offset)
{	static char _rtc_temp_str[5]={0};
	char s[8]={0};
	UWORD __temp=0;

	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(MSB_TEMP);
	i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
	__temp=i2c_readAck();
	__temp<<=8;
	__temp|=i2c_readNak();
	i2c_stop();
	__temp>>=6;
	_rtc_temp=__temp*.25;
	strcpy(_rtc_temp_str, insert_char( trimm_string(' ',itoa( (UWORD)((_rtc_temp+offset)*10),  s, 10), 3 ), ',', 2));
	return _rtc_temp_str;
}


char *wt_str_rtc(void)
{	static char _rtc_wt_str[11];

	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(DAY);

	i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
	strcpy(_rtc_wt_str, strWochentagname(i2c_readNak()&7));
	i2c_stop();
	return _rtc_wt_str;
}


void alarm1_set_rel_sec_max65536(UWORD plus_sekunden)
{	ULONG absolut=0; UBYTE al1_reg[5]={0};// ULONG temp_byte;
// (1<<7)  Alarm_n_M Bits immer auf 7 shiften
	alarm1_clr_rtc();
	read_rtcTD();

//_rtc_t[2]-[0] hh:mm:ss
//_rtc_d[2]-[0] yy:mo:dd

	absolut=_rtc_d[3];		// wday
	absolut*=24;				// in Stunden
	absolut+=_rtc_t[2];		// + Stunden
	absolut*=60;				// in min
	absolut+=_rtc_t[1];		// + min
	absolut*=60;				// in sec
	absolut+=_rtc_t[0];		// + sec
	absolut+=plus_sekunden;	// +plus_sekunden

	al1_reg[2]=(unsigned char)((ULONG)absolut/(ULONG)3600);
	absolut-=(al1_reg[2]*3600); al1_reg[2]=int2bcd(al1_reg[2]);
	al1_reg[1]=(unsigned char)((UWORD)absolut/(UWORD)60);
	absolut-=(al1_reg[1]*60); al1_reg[1]=int2bcd(al1_reg[1]);
	al1_reg[0]=(unsigned char)absolut; al1_reg[0]=int2bcd(al1_reg[0]);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(7);                       // write address = 7
   i2c_write(al1_reg[0]);                    // 
   i2c_write(al1_reg[1]);                    // 
   i2c_write(al1_reg[2]);                    // 
   i2c_write(al1_reg[3]);                    // 
   i2c_stop();     
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
  	i2c_write(DS3231_ControlReg);       // write address = DS3231_ControlReg
	i2c_write( (1<<A1IE)|(1<<INTCN)	);
	i2c_stop();
/*	#define DS3231_ControlReg	0xe
	#define EOSC 					7
	#define BBSQW 					6
	#define CONV 					5
	#define RS2 					4
	#define RS1						3
	#define INTCN					2
	#define A2IE					1
	#define A1IE					0
	#define DS3231_StatusReg	0xf
	#define OSF						7 
	#define EN32kHz				3 
	#define BSY						2 
	#define A2F						1 
	#define A1F						0 */
}


void alarm1_set_abs_rtc(UBYTE sekunde, UBYTE minute, UBYTE stunde, UBYTE tag)
{	
	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(7);                       // write address = 0
   i2c_write(int2bcd(sekunde&0x7f));	//  | A1M1
   i2c_write(int2bcd(minute&0x7f));		//  | A1M2
   i2c_write(int2bcd(stunde&0x3f));		//  | A1M3
   i2c_write(int2bcd(tag&0x3f));			//  | A1M4
   i2c_stop();     

	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
  	i2c_write(DS3231_ControlReg);       // write address = DS3231_ControlReg
	i2c_write( (1<<A1IE)|(1<<INTCN)	);
	i2c_stop();
}


void alarm2_set_rel_min_max65536(UWORD plus_minuten)
{	ULONG absolut=0; unsigned char al2_reg[5]={0};
// (1<<7)  Alarm_n_M Bits immer auf 7 shiften
	alarm2_clr_rtc();
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
	i2c_write(0);                       // write address = 0
	i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
	al2_reg[0]=i2c_readAck()&0x7f;
	al2_reg[1]=i2c_readAck()&0x7f;
	al2_reg[2]=i2c_readAck()&0x3f;
	al2_reg[3]=i2c_readAck()&7;			// 0x03
	al2_reg[3]=i2c_readNak()&0x3f;		// 0x03
	i2c_stop();
	absolut=plus_minuten*60;
	absolut+=bcdtoint(al2_reg[0]); absolut+=bcdtoint(al2_reg[1])*60; absolut+=(ULONG)((ULONG)bcdtoint(al2_reg[2])*(ULONG)3600);
/*	if(absolut>604800)
		; // date
	else
		; / / day */
	al2_reg[2]=(unsigned char)((ULONG)absolut/(ULONG)3600);
	absolut-=(al2_reg[2]*3600); al2_reg[2]=int2bcd(al2_reg[2]);
	al2_reg[1]=(unsigned char)((UWORD)absolut/(UWORD)60);
	absolut-=(al2_reg[1]*60); al2_reg[1]=int2bcd(al2_reg[1]);
	al2_reg[0]=(unsigned char)absolut; al2_reg[0]=int2bcd(al2_reg[0]);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(0x0b);                       // write address = 7
   i2c_write(al2_reg[1]);                    // 
   i2c_write(al2_reg[2]);                    // 
   i2c_write(al2_reg[3]);                    // 
   i2c_stop();     
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
  	i2c_write(DS3231_ControlReg);       // write address = DS3231_ControlReg
	i2c_write( (1<<A2IE)|(1<<INTCN)	);
	i2c_stop();
/*	#define DS3231_ControlReg	0xe
	#define EOSC 					7
	#define BBSQW 					6
	#define CONV 					5
	#define RS2 					4
	#define RS1						3
	#define INTCN					2
	#define A2IE					1
	#define A1IE					0
	#define DS3231_StatusReg	0xf
	#define OSF						7 
	#define EN32kHz				3 
	#define BSY						2 
	#define A2F						1 
	#define A1F						0 */
}


void alarm2_set_abs_rtc(UBYTE minute, UBYTE stunde, UBYTE tag)
{		
	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(0xb);                     // write address = 0
   i2c_write(int2bcd(minute&0x7f));		//  | A2M1
   i2c_write(int2bcd(stunde&0x3f));		//  | A2M2
   i2c_write(int2bcd(tag&0x3f));			//  | A2M3
   i2c_stop();     

	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
  	i2c_write(DS3231_ControlReg);       // write address = DS3231_ControlReg
	i2c_write( (1<<A2IE)|(1<<INTCN)	);
	i2c_stop();
}


UBYTE sekunde_rtc(void)
{	static char sekunde;
	while(_i2c_busy);
	i2c_start_wait(DS3231+I2C_WRITE);	// set device address and write mode
   i2c_write(0);                       // write address = 0
	i2c_rep_start(DS3231+I2C_READ);     // set device address and read mode
	sekunde=bcdtoint(i2c_readNak()&0x7f);
	i2c_stop();
	return sekunde;
}

